import { Plugin } from 'ckeditor5/src/core';
import InsertQBankMediaCommand from './insertqbankmediacommand';

export default class QBankMediaEditing extends Plugin {


  init() {
    this._defineSchema();
    this._defineConverters();
    this.editor.commands.add(
      'insertQBankMedia',
      new InsertQBankMediaCommand(this.editor),
    );
  }

  //Plugin Schemas
  _defineSchema() {
    const schema = this.editor.model.schema;

    schema.register('qbankMedia', {
      isObject: true,
      allowWhere: '$block',
      allowAttributes: ['alt', 'src']
    });

  }

  //Convert component into HTML
  _defineConverters() {
    const { conversion } = this.editor;

    //Store model data including attributes into view
    conversion.for('dataDowncast').elementToElement({
      model: 'qbankMedia',
      view: {
        name: 'img'
      },
    }).attributeToAttribute({
      view: {
        name: 'img',
        key: 'src'
      },
      model: 'src'
    });


    //Convert to HTML tag
    conversion.for('editingDowncast').elementToElement({
      model: 'qbankMedia',
      view: (modelElement, { writer: viewWriter }) => {
        const media = viewWriter.createEmptyElement('img', {
          src: modelElement.getAttribute('src')
        });

        return media;
      },
    });

  }
}
